@@ -317,7 +317,6 @@ g_tls_certificate_gnutls_verify (GTlsCertificate     *cert,
   guint num_certs, i;
   gnutls_x509_crt_t *chain;
   GTlsCertificateFlags gtls_flags;
-  time_t t, now;
 
   cert_gnutls = G_TLS_CERTIFICATE_GNUTLS (cert);
   num_certs = 0;
@@ -349,8 +348,7 @@ g_tls_certificate_gnutls_verify (GTlsCertificate     *cert,
       ca = priv->cert;
       status = gnutls_x509_crt_list_verify (chain, num_certs,
                                             &ca, 1,
-                                            NULL, 0,
-                                            GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT,
+                                            NULL, 0, 0,
                                             &gnutls_flags);
       if (status != 0)
         {
@@ -363,21 +361,6 @@ g_tls_certificate_gnutls_verify (GTlsCertificate     *cert,
   else
     gtls_flags = 0;
 
-  /* We have to check these ourselves since gnutls_x509_crt_list_verify
-   * won't bother if it gets an UNKNOWN_CA.
-   */
-  now = time (NULL);
-  for (i = 0; i < num_certs; i++)
-    {
-      t = gnutls_x509_crt_get_activation_time (chain[i]);
-      if (t == (time_t) -1 || t > now)
-        gtls_flags |= G_TLS_CERTIFICATE_NOT_ACTIVATED;
-
-      t = gnutls_x509_crt_get_expiration_time (chain[i]);
-      if (t == (time_t) -1 || t < now)
-        gtls_flags |= G_TLS_CERTIFICATE_EXPIRED;
-    }
-
   g_free (chain);
 
   if (identity)
@@ -389,11 +372,11 @@ g_tls_certificate_gnutls_verify (GTlsCertificate     *cert,
 static void
 g_tls_certificate_gnutls_real_copy (GTlsCertificateGnutls    *gnutls,
                                     const gchar              *interaction_id,
-                                    gnutls_retr2_st          *st)
+                                    gnutls_pcert_st         **pcert,
+                                    unsigned int             *pcert_length,
+                                    gnutls_privkey_t         *pkey)
 {
   GTlsCertificateGnutls *chain;
-  gnutls_x509_crt_t cert;
-  gnutls_datum_t data;
   guint num_certs = 0;
   size_t size = 0;
   int status;
@@ -409,14 +392,18 @@ g_tls_certificate_gnutls_real_copy (GTlsCertificateGnutls    *gnutls,
       chain = priv->issuer;
     }
 
-  st->ncerts = 0;
-  st->cert.x509 = gnutls_malloc (sizeof (gnutls_x509_crt_t) * num_certs);
+  *pcert_length = 0;
+  *pcert = gnutls_malloc (sizeof (gnutls_pcert_st) * num_certs);
+  if (*pcert == NULL)
+    g_error ("%s: out of memory", __FUNCTION__);
 
   /* Now do the actual copy of the whole chain. */
   chain = gnutls;
   while (chain != NULL)
     {
       GTlsCertificateGnutlsPrivate *priv = g_tls_certificate_gnutls_get_instance_private (chain);
+      gnutls_x509_crt_t cert;
+      gnutls_datum_t data;
 
       gnutls_x509_crt_export (priv->cert, GNUTLS_X509_FMT_DER,
                               NULL, &size);
@@ -430,8 +417,9 @@ g_tls_certificate_gnutls_real_copy (GTlsCertificateGnutls    *gnutls,
       g_warn_if_fail (status == 0);
       g_free (data.data);
 
-      st->cert.x509[st->ncerts] = cert;
-      st->ncerts++;
+      gnutls_pcert_import_x509 (*pcert + *pcert_length, cert, 0);
+      gnutls_x509_crt_deinit (cert);
+      (*pcert_length)++;
 
       chain = priv->issuer;
     }
@@ -441,13 +429,22 @@ g_tls_certificate_gnutls_real_copy (GTlsCertificateGnutls    *gnutls,
 
       if (priv->key != NULL)
         {
-          gnutls_x509_privkey_init (&st->key.x509);
-          gnutls_x509_privkey_cpy (st->key.x509, priv->key);
-          st->key_type = GNUTLS_PRIVKEY_X509;
+          gnutls_x509_privkey_t x509_privkey;
+          gnutls_privkey_t privkey;
+
+          gnutls_x509_privkey_init (&x509_privkey);
+          gnutls_x509_privkey_cpy (x509_privkey, priv->key);
+
+          gnutls_privkey_init (&privkey);
+          gnutls_privkey_import_x509 (privkey, x509_privkey, GNUTLS_PRIVKEY_IMPORT_COPY);
+          *pkey = privkey;
+          gnutls_x509_privkey_deinit (x509_privkey);
+        }
+      else
+        {
+          *pkey = NULL;
         }
     }
-
-  st->deinit_all = TRUE;
 }
 
 static void
@@ -522,14 +519,23 @@ g_tls_certificate_gnutls_has_key (GTlsCertificateGnutls *gnutls)
 }
 
 void
-g_tls_certificate_gnutls_copy  (GTlsCertificateGnutls *gnutls,
-                                const gchar           *interaction_id,
-                                gnutls_retr2_st       *st)
+g_tls_certificate_gnutls_copy  (GTlsCertificateGnutls  *gnutls,
+                                const gchar            *interaction_id,
+                                gnutls_pcert_st       **pcert,
+                                unsigned int           *pcert_length,
+                                gnutls_privkey_t       *pkey)
 {
   g_return_if_fail (G_IS_TLS_CERTIFICATE_GNUTLS (gnutls));
-  g_return_if_fail (st != NULL);
+  g_return_if_fail (pcert != NULL);
+  g_return_if_fail (pcert_length != NULL);
+  g_return_if_fail (pkey != NULL);
   g_return_if_fail (G_TLS_CERTIFICATE_GNUTLS_GET_CLASS (gnutls)->copy);
-  G_TLS_CERTIFICATE_GNUTLS_GET_CLASS (gnutls)->copy (gnutls, interaction_id, st);
+
+  G_TLS_CERTIFICATE_GNUTLS_GET_CLASS (gnutls)->copy (gnutls,
+                                                     interaction_id,
+                                                     pcert,
+                                                     pcert_length,
+                                                     pkey);
 }
 
 static const struct {
